# load libraries
library(rags2ridges)

# load data
load("lungTCGAdata.Rdata") 

# cancer group: determine optimal penalty parameter and estimate the precision
Y0            <- cbind(t(GE0), t(CN0))
optLambdaGrpC <- optPenalty.kCVauto(Y0, 10^(-6), 100, fold=5, 
                                    target=matrix(0, ncol(Y0), ncol(Y0))) 
PplainC       <- optLambdaGrpC[[2]] 

# metastasis group: determine optimal penalty parameter and estimate the precision
Y1            <- cbind(t(GE1), t(CN1))
optLambdaGrpM <- optPenalty.kCVauto(Y0, 10^(-2), 10^(10), 1/10, fold=5, 
                 target=PplainC) 
PplainM       <- optLambdaGrpM[[2]] 

# set parameters
p      <- nrow(GE0)
gNames <- rownames(GEdata)

# add names to rows and columns
colnames(PplainC) <- rownames(PplainC) <- c(gNames, paste(gNames, "cn", sep=""))
colnames(PplainM) <- rownames(PplainM) <- c(gNames, paste(gNames, "cn", sep=""))

# heatmaps of estimated partial correlation matrices
edgeHeat(pcor(optLambdaGrpC[[1]]), diag=FALSE, 
	 main="estimated partial correlation matrix, cancer, null target")
edgeHeat(pcor(optLambdaGrpM[[2]]), diag=FALSE, 
	 main="estimated partial correlation matrix, metastasis, informative target")

# for network visualization: sparsify the left-upper block of the precision matrices
PplainC0 <- sparsify(pcor(PplainC)[1:p, 1:p], "localFDR", FDRcut=0.8)
PplainM0 <- sparsify(pcor(PplainM)[1:p, 1:p], "localFDR", FDRcut=0.8)
colnames(PplainM0) <- rownames(PplainM0) <- gNames
colnames(PplainC0) <- rownames(PplainC0) <- gNames

# plot networks of both groups
Ugraph(PplainC0, type="fancy", lay="layout_with_fr", prune=TRUE, Vsize=12, 
       Vcex=0.6, Vcol="pink", VLcolor="black", VBcolor="purple", main="Cancer/Metastasis")
Ugraph(PplainM0, type="fancy", lay="layout_with_fr", prune=TRUE, Vsize=12, Vcex=0.6, 
       Vcol="lightgreen", VLcolor="black", VBcolor="darkgreen", main="Cancer/Metastasis")


