# load libraries
library(mvtnorm)
library(rags2ridges)
library(Matrix)
library(Rcpp)

# load implementation of 
# generalized ridge and lasso precision estimators 
sourceCpp("generalizedPfunctions.cpp")

# set dimension and sample size
p <- 100
n <- 10

# penalty parameter values
lambda <- 1 * matrix(1, p, p)

# generate banded precision matrix
diags <- list(rep(1, p), rep(0.5, p-1), rep(0.25, p-2), rep(0.1, p-3))
Omega <- as.matrix(bandSparse(p, k = -c(0:3), diag = c(diags), symm=TRUE))
Omega[Omega==0] <- 0.1

# define a banded target
target <- matrix(0, p, p)
target[1,] <- Omega[1,]
target[,1] <- Omega[,1]


###################################################################################
# loss comparison
###################################################################################

# administration object
accuracy <- matrix(nrow=0, ncol=2)
colnames(accuracy) <- c("accFrobG", "accFrobR")

# evaluate accuracy many times
for (k in 1:100){
	# generate data 
	Y <- rmvnorm(n, sigma=solve(Omega))
	S <- covML(Y)

	# estimate precision with regular ridge
	lambdaOptR <- optPenalty.kCVauto(Y, lambdaMin=0.000001, lambdaMax=100000, lambdaInit=1, target=target)$optLambda
	PridgeR    <- ridgeP(S, lambda=lambdaOptR, target=target)

	# make k-folds as list
	fold         <- 5
	fold         <- max(min(ceiling(fold), nrow(Y)), 2)
	fold         <- rep(1:fold, ceiling(nrow(Y)/fold))[1:nrow(Y)]
	shuffle      <- sample(1:nrow(Y), nrow(Y))
	folds        <- split(shuffle, as.factor(fold))
	names(folds) <- NULL

	# fit for the generalized ridge
	CVs <- numeric()
	lambdas <- exp(seq(log(0.001), log(10), length.out=100))
	for (m in 1:length(lambdas)){	
		print(m)
		# define a blocked penalty matrix
		lambdaMat <- matrix(lambdas[m], p, p)
		lambdaMat[1,] <- 10^10
		lambdaMat[,1] <- 10^10
		CVs <- c(CVs, .armaKcvlossR(lambdaMat, Y, target, folds, 100, 10^(-10)))	
	}	
	lambdaOpt <- lambdas[which.min(CVs)]

	# define a blocked penalty matrix
	lambdaMat     <- matrix(lambdaOpt, p, p)
	lambdaMat[1,] <- 10^10
	lambdaMat[,1] <- 10^10
	PridgeG       <- .armaRidgePgen(S, lambdaMat, target, nInit=100, minSuccDiff=10^(-10))

	# calculate accuracies
	slh <- matrix(0, p, p)
	slh[1,] <- 1
	slh[,1] <- 1
	ids <- which(slh == 0, arr.ind=TRUE)
	accFrobG <- sum(((PridgeG -  Omega)[ids])^2)
	accFrobR <- sum(((PridgeR -  Omega)[ids])^2)
	accuracy <- rbind(accuracy, c(accFrobG, accFrobR))
}

